;nyquist plug-in
;version 4
;type analyze
;name "NotSNRcalcPluginV1"
;maxlen 2143260000
;debugflags trace
;author "Steve Daulton"
;release 2.4.2-1
;copyright "Released under terms of the GNU General Public License version 2"

;This is a mildly modified version of "ACX Check" renamed "NotSNRcalcPluginV1" rev2 intended for use
;with the Audacity macro "SNRisoITUmacroV1".
;While "NotSNRcalcPluginV1" doesn't on its own calculate an SNR value, "SNRisoITUmacroV1" does.
;"SNRisoITUmacroV1" calculates an ISO226/ITU468 weighted SNR value as described in
;https://xeno-canto.org/article/275

;This is revision 6

;Revision 6 uses the new SNR interval sets for classifying quality A-E

;Revision 6  SNR intervals:                   Revision 1-5 SNR intervals:
;A:                  SNR ≥ +50dB(ISO/ITU)                      SNR ≥ +50dB(ISO/ITU)
;B: +35dB(ISO/ITU) ≤ SNR ≤ +50dB(ISO/ITU)     +30dB(ISO/ITU) ≤ SNR ≤ +50dB(ISO/ITU)
;C: +20dB(ISO/ITU) ≤ SNR ≤ +35dB(ISO/ITU)     +10dB(ISO/ITU) ≤ SNR ≤ +30dB(ISO/ITU)
;D:  +5dB(ISO/ITU) ≤ SNR ≤ +20dB(ISO/ITU)     -10dB(ISO/ITU) ≤ SNR ≤ +10dB(ISO/ITU)
;E:  +5dB(ISO/ITU) ≥ SNR                      -10dB(ISO/ITU) ≥ SNR 

;Revision 6 presents assessed SNR values <20dB(ISO/ITU) as the recording having a sound quality being
;either one of D or E. This as assessed SNR values <20dB(ISO/ITU) have a low degree of precision.

;Revision 5 presents assessed SNR values <20dB(ISO/ITU) as the recording having a sound quality being
;either one of C, D or E. This as assessed SNR values <20dB(ISO/ITU) have a low degree of precision.

;Revision 4 correctly presents a positive SNR(ISO/ITU) value. Previous revisions
;of NotSNRcalcPluginV1.ny presented a negative SNR(ISO/ITU) value which was just an artefact, the true
;value was positive.

;Revision 3 presents an integer value SNR(ISO/ITU), contrary to previous revisions which presented
;data with two decimals (overkill).

;Revision 2 presents only SNR(ISO/ITU), contrary to revision 1 which also presented redundant data on
;'peak level' and 'RMS level', both of which were nonsense artefact remnants from 'ACX check'. 

(defun getfloor ()
  ;; Calculate RMS where rate=10 Hz, window-size=0.4 seconds.
  ;; Return the lowest 0.4 to 0.5 s in the selection.
  (let ((floor 999)
        (window-size (round (* 0.4 *sound-srate*)))
        samples)
    (setf *track* (s-rms *track* 10 window-size))
    ;; Calculate new length in samples without retaining samples in RAM.
    (setf samples (truncate (* len (/ (snd-srate *track*) *sound-srate*))))
    (do ((val (snd-fetch *track*) (snd-fetch *track*))
         (count samples (1- count)))
        ((< count 4) floor) ;stop at last full window.
      (setf floor (min floor val)))))


(defun s-rms (sig &optional (rate 100.0) window-size)
  ;;; Like RMS function but also supports stereo sounds
  ;;; Stereo RMS is the root mean of all (samples ^ 2) [both channels]
  (when (soundp sig)
    (if window-size
        (return-from s-rms (rms sig rate window-size))
        (return-from s-rms (rms sig rate))))
  (let (left-ms right-ms rslt step-size)
    (setf step-size (round (/ (snd-srate (aref sig 0)) rate)))
    (unless window-size
      (setf window-size step-size))
    (setf (aref sig 0) (mult (aref sig 0)(aref sig 0)))
    (setf (aref sig 1) (mult (aref sig 1)(aref sig 1)))
    (setf left-ms (snd-avg (aref sig 0) window-size step-size OP-AVERAGE))
    (setf right-ms (snd-avg (aref sig 1) window-size step-size OP-AVERAGE))
    (s-sqrt (mult 0.5 (sum left-ms right-ms)))))


(defun track-rms ()
  ;;; Teturn the RMS of *track*.
  ;;; Stereo RMS is the root mean of all (samples ^ 2) [both channels]
  (let ((rms (get '*selection* 'rms)))
    (if (arrayp rms)
        (let ((left-mean-sq (* (aref rms 0)(aref rms 0)))
              (right-mean-sq (* (aref rms 1)(aref rms 1))))
          (sqrt (/ (+ left-mean-sq right-mean-sq) 2.0)))
    rms)))


(defun check-peak (peak)
  ;;; Return Pass, fail or warning.
  ;;; Peak too low is unlikely, but indicate the problem.
  (if (> peak -2.99)
      ""
      (if (< peak -6)
          ""
          "")))


(defun check-rms (rms)
  ;;; Return Pass or Fail.
  (if (> rms -17.99)
      ""
      (if (< rms -23)
          ""
          "")))


(defun check-floor (floor)
  ;;; Return Pass, Fail or warning.
  ;;; Room tone is expected - warn if extremely low.
  (if (> floor -19.99)
      "Quality D or E"
      (if (> floor -34.99)
          "Quality C"
          (if (> floor -49.99)
	       "Quality B"
	       "Quality A"))))


(defun check-rate ()
  ;; Return warning or empty string.
  ;;; ACX require 44100 sample rate.
  (if (/= (round *sound-srate*) 44100)
      ""
      ""))


(defun check-length ()
  ;;; Return warning or empty string.
  ;;; ACX require running time no longer than 120 minutes.
  (let* ((start (get '*track* 'start-time))
         (end (get '*track* 'end-time))
         (dur (- end start)))
    (if (> dur (* 60 120))
        "\n\nWarning: ACX require running time no longer than 120 minutes"
        "")))


(defun selection-too-long ()
  ;; Return true if selection is more than ";maxlen" (13.5 hours at 44100 Hz)
  (let ((start (get '*selection* 'start))
        (end (get '*selection* 'end)))
    (> (truncate (* *sound-srate* (- end start))) len)))


(cond
  ((<= (/ len *sound-srate*) 0.5) ;get length without retaining samples in RAM.
    "Error.\nSelection must be more than 1/2 second.")
  ((selection-too-long)
    "Error.\nSelection too long.\n\nMaximum selection length is\n13.5 hours at 44100 Hz sample rate.")
  (t
    (setf *float-format* "%.0f")
    (let ((peak (linear-to-db (get '*selection* 'peak-level)))
          (rms (linear-to-db (track-rms)))
          (floor (linear-to-db (getfloor))))
      (format nil "SNR:  ~a dB(ISO/ITU)  ~a~%~%~
DELETE fade-in, fade-out and silent parts first!

"
              (- floor)
              (check-floor floor)
))))
